﻿
namespace Microsoft.Kinect.CorreccioEsportiva
{
    using System;
    using System.IO;
    using System.Windows;
    using System.Windows.Media;
    using Microsoft.Kinect;
  

    /// Interaction logic for MainWindow.xaml
    public partial class MainWindow : Window
    {
        /// Ample del dibuix de sortida (quadre negre on es dibuixarà esquelet, s'amplia cap a la dreta)
        private const float RenderWidth = 640.0f;

        /// Altura del dibuix de sortida
        private const float RenderHeight = 480.0f;

        /// Gruix de les unions
        private const double JointThickness = 3;

        /// Gruix de la elipsis central
        private const double BodyCenterThickness = 10;

        /// Gruix dels rectangles per indicar que esquelet surt del quadre.
        private const double ClipBoundsThickness = 10;

        /// Color de la circunferencia/eclipse quan esquelet no estigui traquejat
        private readonly Brush centerPointBrush = Brushes.Blue;

        /// Color del pinzell utilitzat per dibuixar Joints que estan siguent trackejades
        private readonly Brush trackedJointBrush = new SolidColorBrush(Color.FromArgb(255, 68, 192, 68));

        /// Color del pinzell utilitzat per dibuixar joints que estan siguent superposades      
        private readonly Brush inferredJointBrush = Brushes.Yellow;

  
        /// Color utilitzat per osos que están siguent trackejats
        private readonly Pen trackedBonePen = new Pen(Brushes.Green, 6);

        /// Color utilitzat per osos que están siguent interferits per altres parts del esquelet      
        private readonly Pen inferredBonePen = new Pen(Brushes.Gray, 1);

        /// Color utilitzat per osos que están siguent trackejats
        private readonly Pen angleBonePen = new Pen(Brushes.Red, 6);

     
        /// Sensor de Kinect ACTIU
        private KinectSensor sensor;
        /// Grup de dibuix per sortida de renderitzat d'esquelet
        private DrawingGroup drawingGroup;
        /// Imatge dibuixada que mostrarem
        private DrawingImage imageSource;


        /// Creem una nova instancia de MainWindow class. (Pantalla de la aplicacio principal)
        public MainWindow()
        {
            InitializeComponent();
        }


        //Mètode d'autocalibració del motor del sensor Kinect DESCARTAT degut a la limitació del motor trobada.
        /*private void AdjustVerticalAngle(Skeleton skeleton)
        {
                //Si esquelet es surt per la part baixa i no per la alta, i angle del sensor vertical es mes gran que -27 graus
            if ((skeleton.ClippedEdges.HasFlag(FrameEdges.Bottom)) && (!skeleton.ClippedEdges.HasFlag(FrameEdges.Top)) && (this.sensor.ElevationAngle > (-27)))
            {
                int verticalAngle=this.sensor.ElevationAngle;
                verticalAngle=verticalAngle-2;
                if (verticalAngle >= (-27))
                {
                    this.sensor.ElevationAngle = verticalAngle;
                }
                else{this.sensor.ElevationAngle = -27;}
            }

            //Si esquelet es surt per la part alta i no per la baixa, i angle del sensor vertical es mes petit que 27 graus
            if ((!skeleton.ClippedEdges.HasFlag(FrameEdges.Bottom)) && (skeleton.ClippedEdges.HasFlag(FrameEdges.Top)) && (this.sensor.ElevationAngle < (27)))
            {
                int verticalAngle = this.sensor.ElevationAngle;
                verticalAngle=verticalAngle+2;
                if (verticalAngle <= (27))
                {
                    this.sensor.ElevationAngle = verticalAngle;
                }
                else { this.sensor.ElevationAngle = 27; }
            }
        }*/


        /// Dibuixa rectangles indicadors a les cantonades del display per mostrar si part de l'esquelet no apareix.
        private static void RectanglesClippedEdges(Skeleton skeleton, DrawingContext drawingContext)
        {
            /// Si esquelet surt per part baixa
            if (skeleton.ClippedEdges.HasFlag(FrameEdges.Bottom))
            {
                ///Dibuixem rectangle amb (Color,null,Nou rectangle=(posicio X0,Y0,Xmax,yMax,Gruix))
                drawingContext.DrawRectangle(
                    Brushes.Red,
                    null,
                    new Rect(0, RenderHeight - ClipBoundsThickness, RenderWidth, ClipBoundsThickness));
            }
            /// Si esquelet surt per part alta
            if (skeleton.ClippedEdges.HasFlag(FrameEdges.Top))
            {
                drawingContext.DrawRectangle(
                    Brushes.Red,
                    null,
                    new Rect(0, 0, RenderWidth, ClipBoundsThickness));
            }
            /// Si esquelet surt per part esquerra
            if (skeleton.ClippedEdges.HasFlag(FrameEdges.Left))
            {
                drawingContext.DrawRectangle(
                    Brushes.Red,
                    null,
                    new Rect(0, 0, ClipBoundsThickness, RenderHeight));
            }
            /// Si esquelet surt per part dreta
            if (skeleton.ClippedEdges.HasFlag(FrameEdges.Right))
            {
                drawingContext.DrawRectangle(
                    Brushes.Red,
                    null,
                    new Rect(RenderWidth - ClipBoundsThickness, 0, ClipBoundsThickness, RenderHeight));
            }
        }


        // Tasques de inicialització del sensor
        // Pantalla principal de la UI carregada -> Event
        private void WindowLoaded(object sender, RoutedEventArgs e)
        {
            // Creem el nou grup de dibuix que utilitzarem per dibuixar.
            this.drawingGroup = new DrawingGroup();
            // Creem una font de imatge que utilitzarem en el nostre control de imatge
            this.imageSource = new DrawingImage(this.drawingGroup);
            // Mostrem el dibuix utilitzant el nostre control de imatge
            Image.Source = this.imageSource;


            // Buscarem tots els posible sensors conectats y utilitzarem el primer conectat.
            // Requereix que el sensor estigui conectat abans de iniciar la aplicació.
            foreach (var potentialSensor in KinectSensor.KinectSensors)
            {
                if (potentialSensor.Status == KinectStatus.Connected)
                {
                    this.sensor = potentialSensor;
                    break;
                }
            }

            // Si existeix un sensor conectat llavors...
            if (null != this.sensor)
            {
                // Activarem el "Skeleton Stream" a aquest sensor per començar a rebre frames del esquelet
                this.sensor.SkeletonStream.Enable();
                // Afegim un event handler que será cridat sempre que hi hagi noves dades de tipues skeleton frames (actualitzarà esquelet rebut) 
                // "The use of the += operator in this context is referred to as subscribing to an event."
                this.sensor.SkeletonFrameReady += this.SensorSkeletonFrameReady;

                // Inicialitzem el sensor!
                try
                {
                    this.sensor.Start();
                }
                catch (IOException)
                {
                    this.sensor = null;
                }
            }
            // Si no es troba sensor, missatge No Kinect Ready
            if (null == this.sensor)
            {
                this.statusBarText.Text =Properties.Resources.NoKinectReady;
            }
        }


        /// Tasques de finalització del sensor
        // Al tencar la finestra principal de la aplicacio -> Event
        private void WindowClosing(object sender, System.ComponentModel.CancelEventArgs e)
        {
            //Si sensor no Null, finalitzarem.
            if (null != this.sensor)
            {
                this.sensor.Stop();
            }
        }

        /// Event Handler pel sensor Kinect cridat a métode "void WinwowLoaded", quan s'ha rebut una frame nova
        private void SensorSkeletonFrameReady(object sender, SkeletonFrameReadyEventArgs e)
        {
            //Creem taula "skeletons", creem nou skeleton a posició 0 de la taula
            Skeleton[] skeletons = new Skeleton[0];

            //Si tenim una nova SkeletonFrame a SkeletonFrameReadyEventArgs e, el guardarem al final de la taula "skeletons"
            using (SkeletonFrame skeletonFrame = e.OpenSkeletonFrame())
            {
                if (skeletonFrame != null)
                {
                    skeletons = new Skeleton[skeletonFrame.SkeletonArrayLength]; //Modifiquem llargada de la taula "skeletons" amb el nombre de esquelets rebuts
                    skeletonFrame.CopySkeletonDataTo(skeletons);  //Copiem les dades obtingudes de la SkeletonFrame obtinguda de "e" al nou skeleton guardat a la taula "skeletons"
                }
            }


            using (DrawingContext dc = this.drawingGroup.Open())
            {
                // Dibuixem un rectangle transparent (fondo negre) per seleccionar el espai de renderitzat
                dc.DrawRectangle(Brushes.Black, null, new Rect(0.0, 0.0, RenderWidth, RenderHeight));

                //Si hi ha algún esquelet guardat a la taula "skeletons"
                if (skeletons.Length != 0)
                {
                    //Per cada Skeleton a la taula "skeletons"...
                    foreach (Skeleton skel in skeletons)
                    {
                        RectanglesClippedEdges(skel, dc); //Cridem métode per dibuixar rectangles a cantonades on esquelet surti de pantalla
                        
                        //Si el skeleton esta siguent Trackejat, cridarem métode DrawBonesAndJoints. 
                        if (skel.TrackingState == SkeletonTrackingState.Tracked)
                        {
                            this.DrawUnionsAndJoints(skel, dc);

                        }
                        //Si només tenim la posició dibuixarem una elipse/circunferencia
                        else if (skel.TrackingState == SkeletonTrackingState.PositionOnly)
                        {
                            dc.DrawEllipse(
                            this.centerPointBrush,
                            null,
                            this.SkeletonPointToScreen(skel.Position),
                            BodyCenterThickness,
                            BodyCenterThickness);
                        }
                    }
                }

                // Per prevenir dibuixar fora de la área de renderitzat, dibuixem nou rectangle a tot el ample de display. Evitem que quadre es mogui a la aplicació
                this.drawingGroup.ClipGeometry = new RectangleGeometry(new Rect(0.0, 0.0, RenderWidth, RenderHeight));
            }
        }


        /// Métode que dibuixara articulacions i joints de tot l' esquelet
        /// skeleton ->esquelet a dibuixar
        /// drawingContext ->Drawing context on dibuixarem
        private void DrawUnionsAndJoints(Skeleton skeleton, DrawingContext drawingContext)
        {

            //Mostrar angles d'articulacions per pantalla
            //Es necessari comprobar sempre si el joint existeix, ja que si no es aixi i val 0, la operacio provocarà un error a la aplicacio.

            // Braç esquerre
            //Mostrar angle Colze esquerre
            if ((JointExist(skeleton, JointType.ShoulderLeft)) && (JointExist(skeleton, JointType.ElbowLeft)) && (JointExist(skeleton, JointType.WristLeft)))
            { MostrarAngleColzeE.Text = string.Format("{0:0.00}", (angleValue3joints(skeleton.Joints[JointType.ShoulderLeft], skeleton.Joints[JointType.ElbowLeft], skeleton.Joints[JointType.WristLeft]))); }
            //Mostrar angle Canell esquerre
            if ((JointExist(skeleton, JointType.ElbowLeft)) && (JointExist(skeleton, JointType.WristLeft)) && (JointExist(skeleton, JointType.HandLeft)))
            { MostrarAngleCanellE.Text = string.Format("{0:0.00}", (angleValue3joints(skeleton.Joints[JointType.ElbowLeft], skeleton.Joints[JointType.WristLeft], skeleton.Joints[JointType.HandLeft]))); }
            //Mostrar angle Espatlla esquerra
            if ((JointExist(skeleton, JointType.ElbowLeft)) && (JointExist(skeleton, JointType.ShoulderLeft)) && (JointExist(skeleton, JointType.ShoulderCenter)))
           { MostrarAngleEspatllaE.Text = string.Format("{0:0.00}", (angleValue3joints(skeleton.Joints[JointType.ElbowLeft], skeleton.Joints[JointType.ShoulderLeft], skeleton.Joints[JointType.ShoulderCenter]))); }

            // Braç dret
            //Mostrar angle Colze dret
            if ((JointExist(skeleton, JointType.ShoulderRight)) && (JointExist(skeleton, JointType.ElbowRight)) && (JointExist(skeleton, JointType.WristRight)))
            { MostrarAngleColzeD.Text = string.Format("{0:0.00}", (angleValue3joints(skeleton.Joints[JointType.ShoulderRight], skeleton.Joints[JointType.ElbowRight], skeleton.Joints[JointType.WristRight]))); }
            //Mostrar angle Canell dret
            if ((JointExist(skeleton, JointType.ElbowRight)) && (JointExist(skeleton, JointType.WristRight)) && (JointExist(skeleton, JointType.HandRight)))
            { MostrarAngleCanellD.Text = string.Format("{0:0.00}", (angleValue3joints(skeleton.Joints[JointType.ElbowRight], skeleton.Joints[JointType.WristRight], skeleton.Joints[JointType.HandRight]))); }
            //Mostrar angle Espatlla dreta
            if ((JointExist(skeleton, JointType.ElbowRight)) && (JointExist(skeleton, JointType.ShoulderRight)) && (JointExist(skeleton, JointType.ShoulderRight)))
            { MostrarAngleEspatllaD.Text = string.Format("{0:0.00}", (angleValue3joints(skeleton.Joints[JointType.ElbowRight], skeleton.Joints[JointType.ShoulderRight], skeleton.Joints[JointType.ShoulderCenter]))); }
            
            //Cama esquerra
            //Mostrar angle Cama esquerre
            if ((JointExist(skeleton, JointType.KneeLeft)) && (JointExist(skeleton, JointType.HipLeft)) && (JointExist(skeleton, JointType.HipCenter)))
            { MostrarAngleCamaE.Text = string.Format("{0:0.00}", (angleValue3joints(skeleton.Joints[JointType.KneeLeft], skeleton.Joints[JointType.HipLeft], skeleton.Joints[JointType.HipCenter]))); }
            //Mostrar angle Genoll esquerre
            if ((JointExist(skeleton, JointType.AnkleLeft)) && (JointExist(skeleton, JointType.KneeLeft)) && (JointExist(skeleton, JointType.HipLeft)))
            { MostrarAngleGenollE.Text = string.Format("{0:0.00}", (angleValue3joints(skeleton.Joints[JointType.AnkleLeft], skeleton.Joints[JointType.KneeLeft], skeleton.Joints[JointType.HipLeft]))); }

            //Cama dreta
            //Mostrar angle Cama dreta
            if ((JointExist(skeleton, JointType.KneeRight)) && (JointExist(skeleton, JointType.HipRight)) && (JointExist(skeleton, JointType.HipCenter)))
            { MostrarAngleCamaD.Text = string.Format("{0:0.00}", (angleValue3joints(skeleton.Joints[JointType.KneeRight], skeleton.Joints[JointType.HipRight], skeleton.Joints[JointType.HipCenter]))); }
            //Mostrar angle Genoll dret
            if ((JointExist(skeleton, JointType.AnkleRight)) && (JointExist(skeleton, JointType.KneeRight)) && (JointExist(skeleton, JointType.HipRight)))
            { MostrarAngleGenollD.Text = string.Format("{0:0.00}", (angleValue3joints(skeleton.Joints[JointType.AnkleRight], skeleton.Joints[JointType.KneeRight], skeleton.Joints[JointType.HipRight]))); }

            //Mostrar angle Coll
            if ((JointExist(skeleton, JointType.Head)) && (JointExist(skeleton, JointType.ShoulderCenter)) && (JointExist(skeleton, JointType.Spine)))
            { MostrarAngleColl.Text = string.Format("{0:0.00}", (angleValue3joints(skeleton.Joints[JointType.Head], skeleton.Joints[JointType.ShoulderCenter], skeleton.Joints[JointType.Spine]))); }
            //Mostrar angle Pelvis
            if ((JointExist(skeleton, JointType.ShoulderCenter)) && (JointExist(skeleton, JointType.Spine)) && (JointExist(skeleton, JointType.HipCenter)))
            { MostrarAnglePelvis.Text = string.Format("{0:0.00}", (angleValue3joints(skeleton.Joints[JointType.ShoulderCenter], skeleton.Joints[JointType.Spine], skeleton.Joints[JointType.HipCenter]))); }



            //Cridem métode DrawBone per cada unió d'articulacions depenent de la seleccio del usuari "trackingOn i slValue
            if (trackingOn.IsChecked == true){
            if (slValue.Value == 0){ //Si volem realitzar correccions posturals i la activitat seleccionada es correr
                //Dibuixar parts Esquena, espatlles, Pelvis
                // Neck: Si coll esta massa doblegat o massa estirat, avisem d'error
                if ((JointExist(skeleton, JointType.Head)) && (JointExist(skeleton, JointType.ShoulderCenter)) && (JointExist(skeleton, JointType.Spine)))
                {
                    if (((180 - margeError.Value) > (angleValue3joints(skeleton.Joints[JointType.Head], skeleton.Joints[JointType.ShoulderCenter], skeleton.Joints[JointType.Spine]))) || ((180 + margeError.Value) < (angleValue3joints(skeleton.Joints[JointType.Head], skeleton.Joints[JointType.ShoulderCenter], skeleton.Joints[JointType.Spine]))))
                    {
                        this.DrawBone2(skeleton, drawingContext, JointType.Head, JointType.ShoulderCenter);
                    }else{
                        this.DrawBone(skeleton, drawingContext, JointType.Head, JointType.ShoulderCenter);
                    }}
                //Spine: Si esquena està massa doblegada, avisem d'error
                if ((JointExist(skeleton, JointType.ShoulderCenter)) && (JointExist(skeleton, JointType.Spine)) && (JointExist(skeleton, JointType.HipCenter)))
                {
                    if ((135 + margeError.Value) < (angleValue3joints(skeleton.Joints[JointType.ShoulderCenter], skeleton.Joints[JointType.Spine], skeleton.Joints[JointType.HipCenter])))
                    {
                        this.DrawBone2(skeleton, drawingContext, JointType.ShoulderCenter, JointType.Spine);
                    }else{
                        this.DrawBone(skeleton, drawingContext, JointType.ShoulderCenter, JointType.Spine);
                    }}
                
                this.DrawBone(skeleton, drawingContext, JointType.ShoulderCenter, JointType.ShoulderLeft);
                this.DrawBone(skeleton, drawingContext, JointType.ShoulderCenter, JointType.ShoulderRight);
                this.DrawBone(skeleton, drawingContext, JointType.Spine, JointType.HipCenter);
                this.DrawBone(skeleton, drawingContext, JointType.HipCenter, JointType.HipLeft);
                this.DrawBone(skeleton, drawingContext, JointType.HipCenter, JointType.HipRight);

                //Dibuixar parts Braç esquerre
                // Left Arm: Si colze esta massa doblegat o massa estirat, avisem d'error
                if ((JointExist(skeleton, JointType.ShoulderLeft)) && (JointExist(skeleton, JointType.ElbowLeft)) && (JointExist(skeleton, JointType.WristLeft))){
                    if (((90 - margeError.Value) > (angleValue3joints(skeleton.Joints[JointType.ShoulderLeft], skeleton.Joints[JointType.ElbowLeft], skeleton.Joints[JointType.WristLeft]))) || ((90 + margeError.Value) < (angleValue3joints(skeleton.Joints[JointType.ShoulderLeft], skeleton.Joints[JointType.ElbowLeft], skeleton.Joints[JointType.WristLeft]))))
                    {
                        this.DrawBone2(skeleton, drawingContext, JointType.ShoulderLeft, JointType.ElbowLeft);
                        this.DrawBone2(skeleton, drawingContext, JointType.ElbowLeft, JointType.WristLeft);
                        this.DrawBone(skeleton, drawingContext, JointType.WristLeft, JointType.HandLeft);
                    }else{
                        this.DrawBone(skeleton, drawingContext, JointType.ShoulderLeft, JointType.ElbowLeft);
                        this.DrawBone(skeleton, drawingContext, JointType.ElbowLeft, JointType.WristLeft);
                        this.DrawBone(skeleton, drawingContext, JointType.WristLeft, JointType.HandLeft);
                }}


                //Dibuixar parts Braç dret
                //Right Arm: Si colze esta massa doblegat o massa estirat, avisem d'error
                if ((JointExist(skeleton, JointType.ShoulderRight)) && (JointExist(skeleton, JointType.ElbowRight)) && (JointExist(skeleton, JointType.WristRight)))
                {
                    if (((90 - margeError.Value) > (angleValue3joints(skeleton.Joints[JointType.ShoulderRight], skeleton.Joints[JointType.ElbowRight], skeleton.Joints[JointType.WristRight]))) || ((90 + margeError.Value) < (angleValue3joints(skeleton.Joints[JointType.ShoulderRight], skeleton.Joints[JointType.ElbowRight], skeleton.Joints[JointType.WristRight]))))
                    {
                        this.DrawBone2(skeleton, drawingContext, JointType.ShoulderRight, JointType.ElbowRight);
                        this.DrawBone2(skeleton, drawingContext, JointType.ElbowRight, JointType.WristRight);
                        this.DrawBone(skeleton, drawingContext, JointType.WristRight, JointType.HandRight);
                    }else{
                        this.DrawBone(skeleton, drawingContext, JointType.ShoulderRight, JointType.ElbowRight);
                        this.DrawBone(skeleton, drawingContext, JointType.ElbowRight, JointType.WristRight);
                        this.DrawBone(skeleton, drawingContext, JointType.WristRight, JointType.HandRight);
                    }}

                //Cama esquerra
                //LeftLeg: Si cama esquerra esta massa adelatada respecte el cos
                if ((JointExist(skeleton, JointType.HipCenter)) && (JointExist(skeleton, JointType.HipLeft)) && (JointExist(skeleton, JointType.KneeLeft)))
                {
                    if ((125 - margeError.Value) > (angleValue3joints(skeleton.Joints[JointType.HipCenter], skeleton.Joints[JointType.HipLeft], skeleton.Joints[JointType.KneeLeft])))
                    {
                        this.DrawBone2(skeleton, drawingContext, JointType.HipLeft, JointType.KneeLeft);
                        this.DrawBone2(skeleton, drawingContext, JointType.KneeLeft, JointType.AnkleLeft);
                        this.DrawBone2(skeleton, drawingContext, JointType.AnkleLeft, JointType.FootLeft);
                    }else{
                        this.DrawBone(skeleton, drawingContext, JointType.HipLeft, JointType.KneeLeft);
                        this.DrawBone(skeleton, drawingContext, JointType.KneeLeft, JointType.AnkleLeft);
                        this.DrawBone(skeleton, drawingContext, JointType.AnkleLeft, JointType.FootLeft);
                    }}


                // Cama dreta
                // Right Leg: Si cama dreta esta massa adelantada respecte el cos
                if ((JointExist(skeleton, JointType.HipCenter)) && (JointExist(skeleton, JointType.HipRight)) && (JointExist(skeleton, JointType.KneeRight)))
                {
                    if ((125 - margeError.Value) > (angleValue3joints(skeleton.Joints[JointType.HipCenter], skeleton.Joints[JointType.HipRight], skeleton.Joints[JointType.KneeRight])))
                    {
                        this.DrawBone2(skeleton, drawingContext, JointType.HipRight, JointType.KneeRight);
                        this.DrawBone2(skeleton, drawingContext, JointType.KneeRight, JointType.AnkleRight);
                        this.DrawBone2(skeleton, drawingContext, JointType.AnkleRight, JointType.FootRight);
                    }else{
                        this.DrawBone(skeleton, drawingContext, JointType.HipRight, JointType.KneeRight);
                        this.DrawBone(skeleton, drawingContext, JointType.KneeRight, JointType.AnkleRight);
                        this.DrawBone(skeleton, drawingContext, JointType.AnkleRight, JointType.FootRight);
                    }}



            }else if (slValue.Value == 1){  //Si volem realitzar correccions posturals i la activitat seleccionada es salt
                //Dibuixar parts Esquena, espatlles, Pelvis
                // Neck: Si coll esta massa doblegat , avisem d'error
                if ((JointExist(skeleton, JointType.Head)) && (JointExist(skeleton, JointType.ShoulderCenter)) && (JointExist(skeleton, JointType.Spine)))
                {
                    if (((180 - margeError.Value) > (angleValue3joints(skeleton.Joints[JointType.Head], skeleton.Joints[JointType.ShoulderCenter], skeleton.Joints[JointType.Spine]))) || ((180 + margeError.Value) < (angleValue3joints(skeleton.Joints[JointType.Head], skeleton.Joints[JointType.ShoulderCenter], skeleton.Joints[JointType.Spine]))))
                    {
                        this.DrawBone2(skeleton, drawingContext, JointType.Head, JointType.ShoulderCenter);
                    }else{
                        this.DrawBone(skeleton, drawingContext, JointType.Head, JointType.ShoulderCenter);
                    }}




                this.DrawBone(skeleton, drawingContext, JointType.ShoulderCenter, JointType.Spine);
                this.DrawBone(skeleton, drawingContext, JointType.Spine, JointType.HipCenter);
                this.DrawBone(skeleton, drawingContext, JointType.HipCenter, JointType.HipLeft);
                this.DrawBone(skeleton, drawingContext, JointType.HipCenter, JointType.HipRight);

                //Dibuixar parts Braç esquerre
                //Espatlles: estàtiques a la posició correcta
                if ((JointExist(skeleton, JointType.ShoulderLeft)) && (JointExist(skeleton, JointType.ShoulderCenter)) && (JointExist(skeleton, JointType.ElbowLeft)))
                {
                    if (((145 - margeError.Value) > (angleValue3joints(skeleton.Joints[JointType.ShoulderCenter], skeleton.Joints[JointType.ShoulderLeft], skeleton.Joints[JointType.ElbowLeft]))) || ((145 + margeError.Value) < (angleValue3joints(skeleton.Joints[JointType.ShoulderCenter], skeleton.Joints[JointType.ShoulderLeft], skeleton.Joints[JointType.ElbowLeft]))))
                    {
                        this.DrawBone2(skeleton, drawingContext, JointType.ShoulderCenter, JointType.ShoulderLeft);
                        this.DrawBone2(skeleton, drawingContext, JointType.ShoulderLeft, JointType.ElbowLeft);
                        this.DrawBone(skeleton, drawingContext, JointType.ElbowLeft, JointType.WristLeft);
                        this.DrawBone(skeleton, drawingContext, JointType.WristLeft, JointType.HandLeft);
                    }else{
                        this.DrawBone(skeleton, drawingContext, JointType.ShoulderCenter, JointType.ShoulderLeft);
                        this.DrawBone(skeleton, drawingContext, JointType.ShoulderLeft, JointType.ElbowLeft);
                        this.DrawBone(skeleton, drawingContext, JointType.ElbowLeft, JointType.WristLeft);
                        this.DrawBone(skeleton, drawingContext, JointType.WristLeft, JointType.HandLeft);
                    }}



                //Dibuixar parts Braç dret
                //Espatlles: estàtiques a la posició correcta
                if ((JointExist(skeleton, JointType.ShoulderRight)) && (JointExist(skeleton, JointType.ShoulderCenter)) && (JointExist(skeleton, JointType.ElbowRight)))
                {
                    if (((145 - margeError.Value) > (angleValue3joints(skeleton.Joints[JointType.ShoulderCenter], skeleton.Joints[JointType.ShoulderRight], skeleton.Joints[JointType.ElbowRight]))) || ((145 + margeError.Value) < (angleValue3joints(skeleton.Joints[JointType.ShoulderCenter], skeleton.Joints[JointType.ShoulderRight], skeleton.Joints[JointType.ElbowRight]))))
                    {
                        this.DrawBone2(skeleton, drawingContext, JointType.ShoulderCenter, JointType.ShoulderRight);
                        this.DrawBone2(skeleton, drawingContext, JointType.ShoulderRight, JointType.ElbowRight);
                        this.DrawBone(skeleton, drawingContext, JointType.ElbowRight, JointType.WristRight);
                        this.DrawBone(skeleton, drawingContext, JointType.WristRight, JointType.HandRight);
                    }
                    else
                    {
                        this.DrawBone(skeleton, drawingContext, JointType.ShoulderCenter, JointType.ShoulderRight);
                        this.DrawBone(skeleton, drawingContext, JointType.ShoulderRight, JointType.ElbowRight);
                        this.DrawBone(skeleton, drawingContext, JointType.ElbowRight, JointType.WristRight);
                        this.DrawBone(skeleton, drawingContext, JointType.WristRight, JointType.HandRight);
                    }
                }

                // Cama esquerra
                // Genoll esquerre: Si el genoll esta completamnt estirat avisar d'error
                if ((JointExist(skeleton, JointType.HipLeft)) && (JointExist(skeleton, JointType.KneeLeft)) && (JointExist(skeleton, JointType.AnkleLeft)))
                {
                    if ((185 - (30 - margeError.Value)) < (angleValue3joints(skeleton.Joints[JointType.HipLeft], skeleton.Joints[JointType.KneeLeft], skeleton.Joints[JointType.AnkleLeft])))
                    {
                        this.DrawBone2(skeleton, drawingContext, JointType.HipLeft, JointType.KneeLeft);
                        this.DrawBone2(skeleton, drawingContext, JointType.KneeLeft, JointType.AnkleLeft);
                        this.DrawBone(skeleton, drawingContext, JointType.AnkleLeft, JointType.FootLeft);
                    }else{
                        this.DrawBone(skeleton, drawingContext, JointType.HipLeft, JointType.KneeLeft);
                        this.DrawBone(skeleton, drawingContext, JointType.KneeLeft, JointType.AnkleLeft);
                        this.DrawBone(skeleton, drawingContext, JointType.AnkleLeft, JointType.FootLeft);
                    }}

                // Cama dreta
                // Genoll dret: Si el genoll esta completamnt estirat avisar d'error
                if ((JointExist(skeleton, JointType.HipRight)) && (JointExist(skeleton, JointType.KneeRight)) && (JointExist(skeleton, JointType.AnkleRight)))
                {
                    if ((185 - (30-margeError.Value)) < (angleValue3joints(skeleton.Joints[JointType.HipRight], skeleton.Joints[JointType.KneeRight], skeleton.Joints[JointType.AnkleRight])))
                    {
                        this.DrawBone2(skeleton, drawingContext, JointType.HipRight, JointType.KneeRight);
                        this.DrawBone2(skeleton, drawingContext, JointType.KneeRight, JointType.AnkleRight);
                        this.DrawBone(skeleton, drawingContext, JointType.AnkleRight, JointType.FootRight);
                    }
                    else
                    {
                        this.DrawBone(skeleton, drawingContext, JointType.HipRight, JointType.KneeRight);
                        this.DrawBone(skeleton, drawingContext, JointType.KneeRight, JointType.AnkleRight);
                        this.DrawBone(skeleton, drawingContext, JointType.AnkleRight, JointType.FootRight);
                    }
                }

            }else if (slValue.Value == 2){  //Si volem realitzar correccions posturals i la activitat seleccionada es boxe
                //Flux:: Si 2 colzes flexionats -> (estem en guardia) -> 2 genolls doblegats
                //       Altrament si 1 colze flexionat i l'altre no -> (estem llençant un cop) -> "Cama adelantada ha d'estar doblegada, l'altra normal" + "Colze estirat ha de tenir un angle <180 i l'altre ha d estar doblegat mantenint la guardia"

                //Dibuixar parts Esquena, espatlles, Pelvis
                this.DrawBone(skeleton, drawingContext, JointType.ShoulderCenter, JointType.ShoulderLeft);
                this.DrawBone(skeleton, drawingContext, JointType.ShoulderCenter, JointType.ShoulderRight);
                this.DrawBone(skeleton, drawingContext, JointType.ShoulderCenter, JointType.Spine);
                this.DrawBone(skeleton, drawingContext, JointType.Spine, JointType.HipCenter);
                this.DrawBone(skeleton, drawingContext, JointType.HipCenter, JointType.HipLeft);
                this.DrawBone(skeleton, drawingContext, JointType.HipCenter, JointType.HipRight);


                if ((JointExist(skeleton, JointType.ShoulderLeft)) && (JointExist(skeleton, JointType.ElbowLeft)) && (JointExist(skeleton, JointType.WristLeft)) && (JointExist(skeleton, JointType.ShoulderRight)) && (JointExist(skeleton, JointType.ElbowRight)) && (JointExist(skeleton, JointType.WristRight)))
                {
                    //Si 2 colzes flexionats (Guardia)
                    if (((90 - margeError.Value) > (angleValue3joints(skeleton.Joints[JointType.ShoulderRight], skeleton.Joints[JointType.ElbowRight], skeleton.Joints[JointType.WristRight]))) && ((90 - margeError.Value) > (angleValue3joints(skeleton.Joints[JointType.ShoulderLeft], skeleton.Joints[JointType.ElbowLeft], skeleton.Joints[JointType.WristLeft]))))
                    {
                        this.DrawBone(skeleton, drawingContext, JointType.Head, JointType.ShoulderCenter);
                        // Cama esquerra
                        // Genoll esquerre: Si el genoll esta completamnt estirat avisar d'error
                        if ((JointExist(skeleton, JointType.HipLeft)) && (JointExist(skeleton, JointType.KneeLeft)) && (JointExist(skeleton, JointType.AnkleLeft)))
                        {
                            if ((185 - (30 - margeError.Value)) < (angleValue3joints(skeleton.Joints[JointType.HipLeft], skeleton.Joints[JointType.KneeLeft], skeleton.Joints[JointType.AnkleLeft])))
                            {
                                this.DrawBone2(skeleton, drawingContext, JointType.HipLeft, JointType.KneeLeft);
                                this.DrawBone2(skeleton, drawingContext, JointType.KneeLeft, JointType.AnkleLeft);
                                this.DrawBone(skeleton, drawingContext, JointType.AnkleLeft, JointType.FootLeft);
                            }
                            else
                            {
                                this.DrawBone(skeleton, drawingContext, JointType.HipLeft, JointType.KneeLeft);
                                this.DrawBone(skeleton, drawingContext, JointType.KneeLeft, JointType.AnkleLeft);
                                this.DrawBone(skeleton, drawingContext, JointType.AnkleLeft, JointType.FootLeft);
                            }
                        }

                        // Cama dreta
                        // Genoll dret: Si el genoll esta completamnt estirat avisar d'error
                        if ((JointExist(skeleton, JointType.HipRight)) && (JointExist(skeleton, JointType.KneeRight)) && (JointExist(skeleton, JointType.AnkleRight)))
                        {
                            if ((185 - (30 - margeError.Value)) < (angleValue3joints(skeleton.Joints[JointType.HipRight], skeleton.Joints[JointType.KneeRight], skeleton.Joints[JointType.AnkleRight])))
                            {
                                this.DrawBone2(skeleton, drawingContext, JointType.HipRight, JointType.KneeRight);
                                this.DrawBone2(skeleton, drawingContext, JointType.KneeRight, JointType.AnkleRight);
                                this.DrawBone(skeleton, drawingContext, JointType.AnkleRight, JointType.FootRight);
                            }
                            else
                            {
                                this.DrawBone(skeleton, drawingContext, JointType.HipRight, JointType.KneeRight);
                                this.DrawBone(skeleton, drawingContext, JointType.KneeRight, JointType.AnkleRight);
                                this.DrawBone(skeleton, drawingContext, JointType.AnkleRight, JointType.FootRight);
                            }
                        }

                        //Dibuixar parts Braç esquerre
                        this.DrawBone(skeleton, drawingContext, JointType.ShoulderLeft, JointType.ElbowLeft);
                        this.DrawBone(skeleton, drawingContext, JointType.ElbowLeft, JointType.WristLeft);
                        this.DrawBone(skeleton, drawingContext, JointType.WristLeft, JointType.HandLeft);

                        //Dibuixar parts Braç dret
                        this.DrawBone(skeleton, drawingContext, JointType.ShoulderRight, JointType.ElbowRight);
                        this.DrawBone(skeleton, drawingContext, JointType.ElbowRight, JointType.WristRight);
                        this.DrawBone(skeleton, drawingContext, JointType.WristRight, JointType.HandRight);



                    } //Si nomes colze dret esta doblegat (cop de puny esquerre)
                    else if ((90 - margeError.Value) > (angleValue3joints(skeleton.Joints[JointType.ShoulderRight], skeleton.Joints[JointType.ElbowRight], skeleton.Joints[JointType.WristRight])))
                    {
                        this.DrawBone(skeleton, drawingContext, JointType.Head, JointType.ShoulderCenter);
                        //Si es dretà, cama esquerra sempre llegerament flexionada
                        if (usuariEsquerra.IsChecked == false)
                        {
                            // Cama esquerra
                            // Genoll esquerre: Si el genoll esta completamnt estirat avisar d'error
                            if ((JointExist(skeleton, JointType.HipLeft)) && (JointExist(skeleton, JointType.KneeLeft)) && (JointExist(skeleton, JointType.AnkleLeft)))
                            {
                                if ((185 - (30 - margeError.Value)) < (angleValue3joints(skeleton.Joints[JointType.HipLeft], skeleton.Joints[JointType.KneeLeft], skeleton.Joints[JointType.AnkleLeft])))
                                {
                                    this.DrawBone2(skeleton, drawingContext, JointType.HipLeft, JointType.KneeLeft);
                                    this.DrawBone2(skeleton, drawingContext, JointType.KneeLeft, JointType.AnkleLeft);
                                    this.DrawBone(skeleton, drawingContext, JointType.AnkleLeft, JointType.FootLeft);
                                }
                                else
                                {
                                    this.DrawBone(skeleton, drawingContext, JointType.HipLeft, JointType.KneeLeft);
                                    this.DrawBone(skeleton, drawingContext, JointType.KneeLeft, JointType.AnkleLeft);
                                    this.DrawBone(skeleton, drawingContext, JointType.AnkleLeft, JointType.FootLeft);
                                }
                            }
         
                            //Cama dreta normal
                            this.DrawBone(skeleton, drawingContext, JointType.HipRight, JointType.KneeRight);
                            this.DrawBone(skeleton, drawingContext, JointType.KneeRight, JointType.AnkleRight);
                            this.DrawBone(skeleton, drawingContext, JointType.AnkleRight, JointType.FootRight);

                        }
                        else
                        { //Si usuari es esquerrà, cama dreta lleugerament flexionada
                            // Cama dreta
                            // Genoll dret: Si el genoll esta completamnt estirat avisar d'error
                            if ((JointExist(skeleton, JointType.HipRight)) && (JointExist(skeleton, JointType.KneeRight)) && (JointExist(skeleton, JointType.AnkleRight)))
                            {
                                if ((185 - (30 - margeError.Value)) < (angleValue3joints(skeleton.Joints[JointType.HipRight], skeleton.Joints[JointType.KneeRight], skeleton.Joints[JointType.AnkleRight])))
                                {
                                    this.DrawBone2(skeleton, drawingContext, JointType.HipRight, JointType.KneeRight);
                                    this.DrawBone2(skeleton, drawingContext, JointType.KneeRight, JointType.AnkleRight);
                                    this.DrawBone(skeleton, drawingContext, JointType.AnkleRight, JointType.FootRight);
                                }
                                else
                                {
                                    this.DrawBone(skeleton, drawingContext, JointType.HipRight, JointType.KneeRight);
                                    this.DrawBone(skeleton, drawingContext, JointType.KneeRight, JointType.AnkleRight);
                                    this.DrawBone(skeleton, drawingContext, JointType.AnkleRight, JointType.FootRight);
                                }
                            }
                            // Cama esquerra normal
                            this.DrawBone(skeleton, drawingContext, JointType.HipLeft, JointType.KneeLeft);
                            this.DrawBone(skeleton, drawingContext, JointType.KneeLeft, JointType.AnkleLeft);
                            this.DrawBone(skeleton, drawingContext, JointType.AnkleLeft, JointType.FootLeft);
                        }

                        // Braç esquerre
                        // Colze esquerre: Si el colze esta completamnt estirat avisar d'error
                        if ((JointExist(skeleton, JointType.ShoulderLeft)) && (JointExist(skeleton, JointType.ElbowLeft)) && (JointExist(skeleton, JointType.WristLeft)))
                        {
                            if ((185 - (30 - margeError.Value)) < (angleValue3joints(skeleton.Joints[JointType.ShoulderLeft], skeleton.Joints[JointType.ElbowLeft], skeleton.Joints[JointType.WristLeft])))
                            {
                                this.DrawBone2(skeleton, drawingContext, JointType.ShoulderLeft, JointType.ElbowLeft);
                                this.DrawBone2(skeleton, drawingContext, JointType.ElbowLeft, JointType.WristLeft);
                            }
                            else
                            {
                                this.DrawBone(skeleton, drawingContext, JointType.ShoulderLeft, JointType.ElbowLeft);
                                this.DrawBone(skeleton, drawingContext, JointType.ElbowLeft, JointType.WristLeft);
                            }
                        }
                        //Canell esquerre: Si canell no esta completament estirat avisar d'error
                        if (JointExist(skeleton, JointType.HandLeft)){
                            if (((180 - margeError.Value) < (angleValue3joints(skeleton.Joints[JointType.ElbowLeft], skeleton.Joints[JointType.WristLeft], skeleton.Joints[JointType.HandLeft]))) && ((180 + margeError.Value) > (angleValue3joints(skeleton.Joints[JointType.ElbowLeft], skeleton.Joints[JointType.WristLeft], skeleton.Joints[JointType.HandLeft]))))
                            {
                                this.DrawBone(skeleton, drawingContext, JointType.WristLeft, JointType.HandLeft);
                            }
                            else
                            {
                                this.DrawBone2(skeleton, drawingContext, JointType.WristLeft, JointType.HandLeft);
                            }

                        }


                        //Dibuixar parts Braç dret
                        this.DrawBone(skeleton, drawingContext, JointType.ShoulderRight, JointType.ElbowRight);
                        this.DrawBone(skeleton, drawingContext, JointType.ElbowRight, JointType.WristRight);
                        this.DrawBone(skeleton, drawingContext, JointType.WristRight, JointType.HandRight);


                    } //Si nomes colze esquerre esta doblegat (cop de puny dret)
                    else if ((90 - margeError.Value) > (angleValue3joints(skeleton.Joints[JointType.ShoulderLeft], skeleton.Joints[JointType.ElbowLeft], skeleton.Joints[JointType.WristLeft])))
                    {
                        this.DrawBone(skeleton, drawingContext, JointType.Head, JointType.ShoulderCenter);
                        //Si es dretà, cama esquerra sempre llegerament flexionada
                        if (usuariEsquerra.IsChecked == false)
                        {
                            // Cama esquerra
                            // Genoll esquerre: Si el genoll esta completamnt estirat avisar d'error
                            if ((JointExist(skeleton, JointType.HipLeft)) && (JointExist(skeleton, JointType.KneeLeft)) && (JointExist(skeleton, JointType.AnkleLeft)))
                            {
                                if ((185 - (30 - margeError.Value)) < (angleValue3joints(skeleton.Joints[JointType.HipLeft], skeleton.Joints[JointType.KneeLeft], skeleton.Joints[JointType.AnkleLeft])))
                                {
                                    this.DrawBone2(skeleton, drawingContext, JointType.HipLeft, JointType.KneeLeft);
                                    this.DrawBone2(skeleton, drawingContext, JointType.KneeLeft, JointType.AnkleLeft);
                                    this.DrawBone(skeleton, drawingContext, JointType.AnkleLeft, JointType.FootLeft);
                                }
                                else
                                {
                                    this.DrawBone(skeleton, drawingContext, JointType.HipLeft, JointType.KneeLeft);
                                    this.DrawBone(skeleton, drawingContext, JointType.KneeLeft, JointType.AnkleLeft);
                                    this.DrawBone(skeleton, drawingContext, JointType.AnkleLeft, JointType.FootLeft);
                                }
                            }

                            //Cama dreta normal
                            this.DrawBone(skeleton, drawingContext, JointType.HipRight, JointType.KneeRight);
                            this.DrawBone(skeleton, drawingContext, JointType.KneeRight, JointType.AnkleRight);
                            this.DrawBone(skeleton, drawingContext, JointType.AnkleRight, JointType.FootRight);

                        }
                        else
                        { //Si usuari es esquerrà, cama dreta lleugerament flexionada
                            // Cama dreta
                            // Genoll dret: Si el genoll esta completamnt estirat avisar d'error
                            if ((JointExist(skeleton, JointType.HipRight)) && (JointExist(skeleton, JointType.KneeRight)) && (JointExist(skeleton, JointType.AnkleRight)))
                            {
                                if ((185 - (30 - margeError.Value)) < (angleValue3joints(skeleton.Joints[JointType.HipRight], skeleton.Joints[JointType.KneeRight], skeleton.Joints[JointType.AnkleRight])))
                                {
                                    this.DrawBone2(skeleton, drawingContext, JointType.HipRight, JointType.KneeRight);
                                    this.DrawBone2(skeleton, drawingContext, JointType.KneeRight, JointType.AnkleRight);
                                    this.DrawBone(skeleton, drawingContext, JointType.AnkleRight, JointType.FootRight);
                                }
                                else
                                {
                                    this.DrawBone(skeleton, drawingContext, JointType.HipRight, JointType.KneeRight);
                                    this.DrawBone(skeleton, drawingContext, JointType.KneeRight, JointType.AnkleRight);
                                    this.DrawBone(skeleton, drawingContext, JointType.AnkleRight, JointType.FootRight);
                                }
                            }
                            // Cama esquerra normal
                            this.DrawBone(skeleton, drawingContext, JointType.HipLeft, JointType.KneeLeft);
                            this.DrawBone(skeleton, drawingContext, JointType.KneeLeft, JointType.AnkleLeft);
                            this.DrawBone(skeleton, drawingContext, JointType.AnkleLeft, JointType.FootLeft);
                        }
         
                        // Braç dret
                        // Colze dret: Si el colze esta completamnt estirat avisar d'error
                        if ((JointExist(skeleton, JointType.ShoulderRight)) && (JointExist(skeleton, JointType.ElbowRight)) && (JointExist(skeleton, JointType.WristRight)))
                        {
                            if ((185 - (30 - margeError.Value)) < (angleValue3joints(skeleton.Joints[JointType.ShoulderRight], skeleton.Joints[JointType.ElbowRight], skeleton.Joints[JointType.WristRight])))
                            {
                                this.DrawBone2(skeleton, drawingContext, JointType.ShoulderRight, JointType.ElbowRight);
                                this.DrawBone2(skeleton, drawingContext, JointType.ElbowRight, JointType.WristRight);
                            }
                            else
                            {
                                this.DrawBone(skeleton, drawingContext, JointType.ShoulderRight, JointType.ElbowRight);
                                this.DrawBone(skeleton, drawingContext, JointType.ElbowRight, JointType.WristRight);
                            }
                        }
                        //Canell dret: Si canell no esta completament estirat avisar d'error
                        if (JointExist(skeleton, JointType.HandRight))
                        {
                            if (((180 - margeError.Value) < (angleValue3joints(skeleton.Joints[JointType.ElbowRight], skeleton.Joints[JointType.WristRight], skeleton.Joints[JointType.HandRight]))) && ((180 + margeError.Value) > (angleValue3joints(skeleton.Joints[JointType.ElbowRight], skeleton.Joints[JointType.WristRight], skeleton.Joints[JointType.HandRight]))))
                            {
                                this.DrawBone(skeleton, drawingContext, JointType.WristRight, JointType.HandRight);
                            }
                            else
                            {
                                this.DrawBone2(skeleton, drawingContext, JointType.WristRight, JointType.HandRight);
                            }

                        }

                        //Dibuixar parts Braç esquerre
                        this.DrawBone(skeleton, drawingContext, JointType.ShoulderLeft, JointType.ElbowLeft);
                        this.DrawBone(skeleton, drawingContext, JointType.ElbowLeft, JointType.WristLeft);
                        this.DrawBone(skeleton, drawingContext, JointType.WristLeft, JointType.HandLeft);


                    } //2 braços estirats, hem perdut la guardia, som vulnerables a nockeig.
                    else
                    {
                        this.DrawBone2(skeleton, drawingContext, JointType.Head, JointType.ShoulderCenter); // 
                        //Si es dretà, cama esquerra sempre llegerament flexionada
                        if (usuariEsquerra.IsChecked == false)
                        {
                            // Cama esquerra
                            // Genoll esquerre: Si el genoll esta completamnt estirat avisar d'error
                            if ((JointExist(skeleton, JointType.HipLeft)) && (JointExist(skeleton, JointType.KneeLeft)) && (JointExist(skeleton, JointType.AnkleLeft)))
                            {
                                if ((185 - (30 - margeError.Value)) < (angleValue3joints(skeleton.Joints[JointType.HipLeft], skeleton.Joints[JointType.KneeLeft], skeleton.Joints[JointType.AnkleLeft])))
                                {
                                    this.DrawBone2(skeleton, drawingContext, JointType.HipLeft, JointType.KneeLeft);
                                    this.DrawBone2(skeleton, drawingContext, JointType.KneeLeft, JointType.AnkleLeft);
                                    this.DrawBone(skeleton, drawingContext, JointType.AnkleLeft, JointType.FootLeft);
                                }
                                else
                                {
                                    this.DrawBone(skeleton, drawingContext, JointType.HipLeft, JointType.KneeLeft);
                                    this.DrawBone(skeleton, drawingContext, JointType.KneeLeft, JointType.AnkleLeft);
                                    this.DrawBone(skeleton, drawingContext, JointType.AnkleLeft, JointType.FootLeft);
                                }
                            }

                            //Cama dreta normal
                            this.DrawBone(skeleton, drawingContext, JointType.HipRight, JointType.KneeRight);
                            this.DrawBone(skeleton, drawingContext, JointType.KneeRight, JointType.AnkleRight);
                            this.DrawBone(skeleton, drawingContext, JointType.AnkleRight, JointType.FootRight);

                        }
                        else
                        { //Si usuari es esquerrà, cama dreta lleugerament flexionada
                            // Cama dreta
                            // Genoll dret: Si el genoll esta completamnt estirat avisar d'error
                            if ((JointExist(skeleton, JointType.HipRight)) && (JointExist(skeleton, JointType.KneeRight)) && (JointExist(skeleton, JointType.AnkleRight)))
                            {
                                if ((185 - (30 - margeError.Value)) < (angleValue3joints(skeleton.Joints[JointType.HipRight], skeleton.Joints[JointType.KneeRight], skeleton.Joints[JointType.AnkleRight])))
                                {
                                    this.DrawBone2(skeleton, drawingContext, JointType.HipRight, JointType.KneeRight);
                                    this.DrawBone2(skeleton, drawingContext, JointType.KneeRight, JointType.AnkleRight);
                                    this.DrawBone(skeleton, drawingContext, JointType.AnkleRight, JointType.FootRight);
                                }
                                else
                                {
                                    this.DrawBone(skeleton, drawingContext, JointType.HipRight, JointType.KneeRight);
                                    this.DrawBone(skeleton, drawingContext, JointType.KneeRight, JointType.AnkleRight);
                                    this.DrawBone(skeleton, drawingContext, JointType.AnkleRight, JointType.FootRight);
                                }
                            }
                            // Cama esquerra normal
                            this.DrawBone(skeleton, drawingContext, JointType.HipLeft, JointType.KneeLeft);
                            this.DrawBone(skeleton, drawingContext, JointType.KneeLeft, JointType.AnkleLeft);
                            this.DrawBone(skeleton, drawingContext, JointType.AnkleLeft, JointType.FootLeft);
                        }

                        //Usuari ja ha comès el error més greu al llençar el cop, no evaluarem la sobre-elongacio del cop de puny
                        //Dibuixar parts Braç esquerre
                        this.DrawBone(skeleton, drawingContext, JointType.ShoulderLeft, JointType.ElbowLeft);
                        this.DrawBone(skeleton, drawingContext, JointType.ElbowLeft, JointType.WristLeft);
                        this.DrawBone(skeleton, drawingContext, JointType.WristLeft, JointType.HandLeft);
                        //Dibuixar parts Braç dret
                        this.DrawBone(skeleton, drawingContext, JointType.ShoulderRight, JointType.ElbowRight);
                        this.DrawBone(skeleton, drawingContext, JointType.ElbowRight, JointType.WristRight);
                        this.DrawBone(skeleton, drawingContext, JointType.WristRight, JointType.HandRight);
                    }
                }


            }
            }else{                    //Si no volem realitzar correcions posturals, simplement dibuixem unions
            //Dibuixar parts Esquena, espatlles, Pelvis
                
            this.DrawBone(skeleton, drawingContext, JointType.Head, JointType.ShoulderCenter);
            this.DrawBone(skeleton, drawingContext, JointType.ShoulderCenter, JointType.ShoulderLeft);
            this.DrawBone(skeleton, drawingContext, JointType.ShoulderCenter, JointType.ShoulderRight);
            this.DrawBone(skeleton, drawingContext, JointType.ShoulderCenter, JointType.Spine);
            this.DrawBone(skeleton, drawingContext, JointType.Spine, JointType.HipCenter);
            this.DrawBone(skeleton, drawingContext, JointType.HipCenter, JointType.HipLeft);
            this.DrawBone(skeleton, drawingContext, JointType.HipCenter, JointType.HipRight);

            //Dibuixar parts Braç esquerre
            this.DrawBone(skeleton, drawingContext, JointType.ShoulderLeft, JointType.ElbowLeft);
            this.DrawBone(skeleton, drawingContext, JointType.ElbowLeft, JointType.WristLeft);
            this.DrawBone(skeleton, drawingContext, JointType.WristLeft, JointType.HandLeft);     

            //Dibuixar parts Braç dret
            this.DrawBone(skeleton, drawingContext, JointType.ShoulderRight, JointType.ElbowRight);
            this.DrawBone(skeleton, drawingContext, JointType.ElbowRight, JointType.WristRight);
            this.DrawBone(skeleton, drawingContext, JointType.WristRight, JointType.HandRight);

            // Cama esquerra
            this.DrawBone(skeleton, drawingContext, JointType.HipLeft, JointType.KneeLeft);
            this.DrawBone(skeleton, drawingContext, JointType.KneeLeft, JointType.AnkleLeft);
            this.DrawBone(skeleton, drawingContext, JointType.AnkleLeft, JointType.FootLeft);

            // Cama dreta
            this.DrawBone(skeleton, drawingContext, JointType.HipRight, JointType.KneeRight);
            this.DrawBone(skeleton, drawingContext, JointType.KneeRight, JointType.AnkleRight);
            this.DrawBone(skeleton, drawingContext, JointType.AnkleRight, JointType.FootRight);
                 
            }
            

            // Articulacions
            foreach (Joint joint in skeleton.Joints)
            {
                //Creem nou pincell per dibuixar articulacions
                Brush drawBrush = null;

                //Si articulacions están trackejadees, pincell d'un color (verd clar per defecte)
                if (joint.TrackingState == JointTrackingState.Tracked)
                {
                    drawBrush = this.trackedJointBrush;                    
                }
                //Si articulacions están superposades, `pincell d'un altre color (groc per defecte)
                else if (joint.TrackingState == JointTrackingState.Inferred)
                {
                    drawBrush = this.inferredJointBrush;                    
                }
                //Si articulacions no estan trackejades ni interposades, dibuixarem circunferencia del color indicat pel pincell drawBrush 
                if (drawBrush != null)
                {
                    drawingContext.DrawEllipse(drawBrush, null, this.SkeletonPointToScreen(joint.Position), JointThickness, JointThickness);
                }
            }
}





        /// Mapeja un esquelet per que aparegui a dins del nostre espai de renderizat i el converteix a un punt. (Quan no el tenim trackejat, només la posició)
        /// <param name="skelpoint">point to map</param>
        /// <returns>mapped point</returns>
        private Point SkeletonPointToScreen(SkeletonPoint skelpoint)
        {
            // Convert point to depth space.  
            // We are not using depth directly, but we do want the points in our 640x480 output resolution.
            DepthImagePoint depthPoint = this.sensor.CoordinateMapper.MapSkeletonPointToDepthPoint(skelpoint, DepthImageFormat.Resolution640x480Fps30);
            return new Point(depthPoint.X, depthPoint.Y);
        }


        private bool JointExist(Skeleton skeleton, JointType jointType0)
        {
            Joint joint0 = skeleton.Joints[jointType0];
            if (joint0.TrackingState == JointTrackingState.NotTracked)
            {
                return false;
            }
            else
            {
                return true;
            }}


        /// <summary>
        /// Dibuixa una linia entre 2 articulacions
        /// </summary>
        /// <param name="skeleton">Skeleton del que dibuixarem linies</param>
        /// <param name="drawingContext">Drawing context on les dibuixarem</param>
        /// <param name="jointType0">Joint on iniciarem la linia</param>
        /// <param name="jointType1">Joint on acabarem la linia</param>
        private void DrawBone(Skeleton skeleton, DrawingContext drawingContext, JointType jointType0, JointType jointType1)
        {
            //Creem 2 noves joints a partir de les entrades per paràmetre
            Joint joint0 = skeleton.Joints[jointType0];
            Joint joint1 = skeleton.Joints[jointType1];

            // Si no podem trobar alguna de les articulacions, sortim.
            if (joint0.TrackingState == JointTrackingState.NotTracked ||
                joint1.TrackingState == JointTrackingState.NotTracked)
            {
                return;
            }

            // No dibuixarem la unió si les dues articulacions están superposades per altres parts del esquelet
            if (joint0.TrackingState == JointTrackingState.Inferred &&
                joint1.TrackingState == JointTrackingState.Inferred)
            {
                return;
            }

            // Asumirem que tots els ossos dibuixats estan interferits, excepte si les 2 articulacions estan trackejades
            Pen drawPen = this.inferredBonePen;
            // Si les 2 articulacions estan trackejades... verd
            if (joint0.TrackingState == JointTrackingState.Tracked && joint1.TrackingState == JointTrackingState.Tracked)
            {
                drawPen = this.trackedBonePen;
            }

            drawingContext.DrawLine(drawPen, this.SkeletonPointToScreen(joint0.Position), this.SkeletonPointToScreen(joint1.Position));
        }


        /// Dibuixa una linia entre 2 articulacions en posició incorrecta (vermell)
        /// <param name="skeleton">Skeleton del que dibuixarem linies</param>
        /// <param name="drawingContext">Drawing context on les dibuixarem</param>
        /// <param name="jointType0">Joint on iniciarem la linia</param>
        /// <param name="jointType1">Joint on acabarem la linia</param>
        private void DrawBone2(Skeleton skeleton, DrawingContext drawingContext, JointType jointType0, JointType jointType1)
        {
            Joint joint0 = skeleton.Joints[jointType0];
            Joint joint1 = skeleton.Joints[jointType1];

            // Si no podem trobar alguna de les articulacions, sortim.
            if (joint0.TrackingState == JointTrackingState.NotTracked ||
                joint1.TrackingState == JointTrackingState.NotTracked)
            {
                return;
            }

            // No dibuixarem la unió si les dues articulacions están superposades per altres parts del esquelet
            if (joint0.TrackingState == JointTrackingState.Inferred &&
                joint1.TrackingState == JointTrackingState.Inferred)
            {
                return;
            }

            // Asumirem que tots els ossos dibuixats estan interferits, excepte si les 2 articulacions estan trackejades
            Pen drawPen = this.inferredBonePen;
            // Si les 2 articulacions estan trackejades... vermell
            if (joint0.TrackingState == JointTrackingState.Tracked && joint1.TrackingState == JointTrackingState.Tracked)
            {
                drawPen = this.angleBonePen;
            }

            drawingContext.DrawLine(drawPen, this.SkeletonPointToScreen(joint0.Position), this.SkeletonPointToScreen(joint1.Position));
        }

        /// Retorna el valor de l'angle a partir de 3 joints.
        /// jointB sempre será la articulació central d'una extremitat o part del cos (la que volem mesurar).
        /// Els vectors surten de la articulacio per tal de que els calculs siguin corretes
        private double angleValue3joints(Joint jointA, Joint jointB, Joint jointC)
        {
            Vector3 a1 = new Vector3(jointA.Position.X,jointA.Position.Y,jointA.Position.Z);
            Vector3 a2 = new Vector3(jointB.Position.X, jointB.Position.Y, jointB.Position.Z);
            Vector3 a3 = new Vector3(jointC.Position.X, jointC.Position.Y, jointC.Position.Z);

            Vector3 b1 = a1 - a2;
            Vector3 b2 = a3 - a2;

            return (b1.Angle(b2))*(180/(Math.PI));
        }

        private void Image_ImageFailed(object sender, ExceptionRoutedEventArgs e)
        {

        }

        private void listBox1_SelectionChanged(object sender, System.Windows.Controls.SelectionChangedEventArgs e)
        {

        }

        private void slValue_ValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {

        }
        private void margeError_ValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {

        }

    }
}